@@ -92,11 +92,16 @@
 (define adecl #f)
 (let* ((code "struct foo { int x, y; } *a, b;\n") (indx 1)
        (code (string-append
-	      "typedef struct foo foo_t;\n"
-	      "struct foo {\n"
-	      "int (*bar)(foo_t*);"
-	      "} x;\n"
-	      "int baz(foo_t*);"
+	      "typedef int *foo_t;\n"
+	      "typedef double hmm_t[3];\n"
+	      "int baz(foo_t (*baz)(hmm_t y));\n"
+	      ))
+       (xcode (string-append
+	      "typedef struct { int x; } foo_t;\n"
+	      "foo_t *y;\n"
+	      ;; expand as
+	      ;; 1) struct struct__foo_t *y;
+	      ;; 2) void *y;
 	      ))
        (indx 2)
        (tree (parse-string code))
@@ -105,22 +110,17 @@
        
        (udict (c99-trans-unit->udict tree))
        ;;(ddict (udict-enums->ddict udict))
-       (udecl '(param-decl (decl-spec-list (type-spec (typename "foo_t")))
-			   (param-declr (abs-declr (pointer)))))
        (udecl (udict-ref udict "baz"))
-       (udecl (sx-ref tree 2))
-       ;;(mspec (udecl->mspec udecl))
        ;;(decl (and=> ((sxpath `((decl ,indx))) tree) car))
-       ;;(xdecl (expand-typerefs decl udict))
+       (xdecl (expand-typerefs udecl udict))
        )
   ;;(display code)
-  ;;(ppsx tree)
-  ;;(pp99 tree)
-  ;;(ppsx udict)
-  (ppsx udecl)
-  (ppsx (expand-typerefs udecl udict '("foo_t")))
+  (pp99 udecl)
+  (display "==\n")
+  (pp99 xdecl)
+  ;;(ppsx mspec)
+  ;;(ppsx (expand-typerefs udecl udict '("foo_t")))
   ;;(ppsx ddict)
-  ;;(display "==\n")
   ;;(set! adecl decl)
   #t)
 
