@@ -1,3 +1,4 @@
+(define redo #f)
 ;; Tmach.scm - javascript
 ;;
 ;; Copyright (C) 2015,2016 Matthew R. Wette
@@ -11,6 +12,10 @@
 (add-to-load-path (string-append (getcwd) "/../../../../examples/"))
 
 (use-modules (nyacc lang javascript mach))
+(when redo
+  (gen-js-files) (system "touch parser.scm")
+  (gen-se-files) (system "touch separser.scm"))
+
 (use-modules (nyacc lang javascript parser))
 (use-modules (nyacc lang javascript pprint))
 (use-modules (nyacc lang javascript compile-tree-il))
@@ -19,17 +24,7 @@
 (use-modules (nyacc export))
 (use-modules (ice-9 pretty-print))
 
-(when #f
-  (gen-js-files)
-  ;;(system "touch parser.scm")
-  )
-
-(when #f
-  (gen-se-files)
-  ;;(system "touch separser.scm")
-  )
-
-(when #f
+(when redo
   (with-output-to-file ",lang.txt"
     (lambda ()
       (pp-lalr-notice js-spec)
@@ -52,10 +47,15 @@
     (lambda () (lalr->bison js-spec))))
 
 (when #t
-  (let ((res (with-input-from-file "ex1.js" dev-parse-js)))
-    (pretty-print res)
-    (compile-tree-il res '() '())
-    ;;(pretty-print-js res)
-    ))
+  (let* ((xargs (cdr (program-arguments)))
+	 (file (if (pair? xargs) (car xargs) ",ex1.js"))
+	 (res (with-input-from-file file dev-parse-js)))
+    (let ((til (compile-tree-il res (current-module) '())))
+      (let ((val (compile til
+			  #:env (current-module)
+			  #:from 'tree-il
+			  #:to 'value)))
+	(simple-format #t "~S\n" val)
+	#t))))
 
 ;; --- last line ---
