@@ -1,19 +1,19 @@
-;;; lang/c/cpp.scm
-;;;
-;;; Copyright (C) 2015-2017 Matthew R. Wette
-;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; lang/c/cpp.scm
+;;
+;; Copyright (C) 2015-2017 Matthew R. Wette
+;;
+;; This program is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by 
+;; the Free Software Foundation, either version 3 of the License, or 
+;; (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;; C preprocessor macro expansion and condition text parse-and-eval
 ;; ref: https://gcc.gnu.org/onlinedocs/gcc-3.0.1/cpp_3.html
@@ -22,12 +22,17 @@
   #:export (
 	    cpp-line->stmt
 	    eval-cpp-cond-text
-	    expand-cpp-macro-ref)
+	    expand-cpp-macro-ref
+	    parse-cpp-expr
+	    find-incl-in-dirl
+	    scan-arg-literal
+	    eval-cpp-expr)
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
   #:use-module (nyacc lang util)
   #:use-module (rnrs arithmetic bitwise)
   #:use-module (ice-9 match)
+  #:use-module (ice-9 textual-ports)
   #:use-module (system base pmatch)
   )
 
@@ -54,7 +59,7 @@
    (else #f)))
 
 (define inline-whitespace (list->char-set '(#\space #\tab)))
-  
+
 ;;.@deffn {Procedure} skip-il-ws ch
 ;; Skip in-line whitespace
 ;; @end deffn
@@ -64,6 +69,19 @@
    ((char-set-contains? inline-whitespace ch) (skip-il-ws (read-char)))
    (else ch)))
 
+;; Not sure about this. We want to turn a list of tokens into a string
+;; with proper escapes.
+(define (esc-c-str str)
+  (list->string
+   (string-fold-right
+    (lambda (ch chl)
+      (case ch
+	((#\\ #\") (cons* #\\ ch chl))
+	(else (cons ch chl))))
+    '() str)))
+
+(define ident-like? (make-ident-like-p read-c-ident))
+
 ;; @deffn {Procedure} read-ellipsis ch
 ;; read ellipsis
 ;; @end deffn
@@ -73,6 +91,28 @@
    ((char=? ch #\.) (read-char) (read-char) "...") ; assumes correct syntax
    (else #f)))
 
+;; @deffn {Procedure} find-incl-in-dirl file dirl next => path
+;; Find path to include file expression, (i.e., @code{<foo.h>} or
+;; @code{"foo.h"}.  If @code{"foo.h"} form look in current directory first.
+;; If @var{next} is true then remove current directory from search path.
+;; @*Refs:
+;; @itemize
+;; @item https://gcc.gnu.org/onlinedocs/cpp/Search-Path.html
+;; @item https://gcc.gnu.org/onlinedocs/cpp/Wrapper-Headers.html
+;; @end itemize
+;; @end deffn
+(define (find-incl-in-dirl file dirl next)
+  (let* ((cid (and=> (port-filename (current-input-port)) dirname))
+	 (file-type (string-ref file 0)) ;; #\< or #\"
+	 (file-name (substring file 1 (1- (string-length file))))
+	 (dirl (if (and cid (char=? #\" file-type)) (cons cid dirl) dirl)))
+    (let iter ((dirl dirl))
+      (if (null? dirl) #f
+	  (if (and next (string=? (car dirl) cid))
+	      (iter (cdr dirl))
+	      (let ((p (string-append (car dirl) "/" file-name)))
+		(if (access? p R_OK) p (iter (cdr dirl)))))))))
+
 ;; @deffn {Procedure} cpp-define
 ;; Reads CPP define from current input and generates a cooresponding sxml
 ;; expression.
@@ -98,7 +138,7 @@
 
   (define (p-rest la) ;; parse rest
     (cond ((eof-object? la) "")
-	  (else (unread-char la) (drain-input (current-input-port)))))
+	  (else (unread-char la) (get-string-all (current-input-port)))))
 
   (let* ((name (read-c-ident (skip-il-ws (read-char))))
 	 (args (or (p-args (read-char)) '()))
@@ -111,12 +151,15 @@
 ;; @deffn {Procedure} cpp-include
 ;; Parse CPP include statement.
 (define (cpp-include)
-  (let* ((beg-ch (skip-il-ws (read-char)))
-	 (end-ch (if (eq? beg-ch #\<) #\> #\"))
-	 (path (let iter ((cl (list beg-ch)) (ch (read-char)))
-		 (if (eq? ch end-ch) (list->string (reverse (cons ch cl)))
-		     (iter (cons ch cl) (read-char))))))
-    `(include ,path)))
+  (define (iter cl ch end-ch)
+    (if (eq? ch end-ch)  (list->string (reverse (cons ch cl)))
+	(iter (cons ch cl) (read-char) end-ch)))
+   (let ((ch (skip-il-ws (read-char))))
+     (cond
+      ((char=? ch #\<) (iter (list #\<) (read-char) #\>))
+      ((char=? ch #\") (iter (list #\") (read-char) #\"))
+      ((read-c-ident ch))
+      (else (throw 'cpp-error "bad include")))))
 
 ;; @deffn {Procedure} cpp-line->stmt line defs => (stmt-type text)
 ;; Parse a line from a CPP statement and return a parse tree.
@@ -133,28 +176,39 @@
   (define (rd-num) (and=> (read-c-num (skip-il-ws (read-char))) cdr))
   (define (rd-rest) (let ((ch (skip-il-ws (read-char))))
 		      (if (not (eof-object? ch)) (unread-char ch))
-		      (drain-input (current-input-port))))
+		      (get-string-all (current-input-port))))
   (with-input-from-string line
     (lambda ()
-      (let ((cmd (string->symbol (read-c-ident (skip-il-ws (read-char))))))
-	(case cmd
-	  ((include) (cpp-include))
-	  ((define) (cpp-define))
-	  ((undef) `(undef ,(rd-ident)))
-	  ((ifdef)
- 	   `(if ,(string-append "defined(" (rd-ident) ")" (rd-rest))))
-	  ((ifndef)
-	   `(if ,(string-append "!defined(" (rd-ident) ")" (rd-rest))))
-	  ((if elif else endif line error pragma) (list cmd (rd-rest)))
-	  (else '(unknown "")))))))
+      (let ((ch (skip-il-ws (read-char))))
+	(cond
+	 ((read-c-ident ch) =>
+	  (lambda (cmds)
+	    (let ((cmd (string->symbol cmds)))
+	      (case cmd
+		((include) `(include ,(cpp-include)))
+		((include_next) `(include-next ,(cpp-include)))
+		((define) (cpp-define))
+		((undef) `(undef ,(rd-ident)))
+		((ifdef)
+		 `(if ,(string-append "defined(" (rd-ident) ")" (rd-rest))))
+		((ifndef)
+		 `(if ,(string-append "!defined(" (rd-ident) ")" (rd-rest))))
+		((if elif else endif line error warning pragma)
+		 (list cmd (rd-rest)))
+		(else
+		 (list 'warning (simple-format #f "unknown CPP: ~S" line)))))))
+	 ((read-c-num ch) => (lambda (num) `(line ,num ,(rd-rest))))
+	 (else (error "missing code")))))))
+	    
 
 (include-from-path "nyacc/lang/c99/mach.d/cpptab.scm")
 (include-from-path "nyacc/lang/c99/mach.d/cppact.scm")
 
 (define cpp-raw-parser
   (make-lalr-parser
-   (list (cons 'len-v cpp-len-v) (cons 'pat-v cpp-pat-v) (cons 'rto-v cpp-rto-v)
-	 (cons 'mtab cpp-mtab) (cons 'act-v cpp-act-v))))
+   (list (cons 'len-v cpp-len-v) (cons 'pat-v cpp-pat-v)
+	 (cons 'rto-v cpp-rto-v) (cons 'mtab cpp-mtab)
+	 (cons 'act-v cpp-act-v))))
 
 (define (cpp-err fmt . args)
   (apply throw 'cpp-error fmt args))
@@ -191,11 +245,11 @@
 ;; Evaluate a tree produced from @code{parse-cpp-expr}.
 ;; The tree passed to this routine is 
 ;; @end deffn
-(define (eval-cpp-expr tree dict)
+(define* (eval-cpp-expr tree dict #:key (inc-dirs '()))
   (letrec
-      ((tx (lambda (tr ix) (list-ref tr ix)))
+      ((tx (lambda (tr ix) (sx-ref tr ix)))
        (tx1 (lambda (tr) (tx tr 1)))
-       (ev (lambda (ex ix) (eval-expr (list-ref ex ix))))
+       (ev (lambda (ex ix) (eval-expr (sx-ref ex ix))))
        (ev1 (lambda (ex) (ev ex 1)))	; eval expr in arg 1
        (ev2 (lambda (ex) (ev ex 2)))	; eval expr in arg 2
        (ev3 (lambda (ex) (ev ex 3)))	; eval expr in arg 3
@@ -203,13 +257,16 @@
 	(lambda (tree)
 	  (case (car tree)
 	    ((fixed) (string->number (cnumstr->scm (tx1 tree))))
-	    ((char) (char->integer (tx1 tree)))
+	    ((char) (char->integer (string-ref (tx1 tree) 0)))
 	    ((defined) (if (assoc-ref dict (tx1 tree)) 1 0))
+	    ((has-include)
+	     (if (find-incl-in-dirl (tx1 tree) inc-dirs #f) 1 0))
+	    ((has-include-next)
+	     (if (find-incl-in-dirl (tx1 tree) inc-dirs #t) 1 0))
 	    ((pre-inc post-inc) (1+ (ev1 tree)))
 	    ((pre-dec post-dec) (1- (ev1 tree)))
 	    ((pos) (ev1 tree))
 	    ((neg) (- (ev1 tree)))
-	    ((bw-not) (bitwise-not (ev1 tree)))
 	    ((not) (if (zero? (ev1 tree)) 1 0))
 	    ((mul) (* (ev1 tree) (ev2 tree)))
 	    ((div) (/ (ev1 tree) (ev2 tree)))
@@ -224,14 +281,20 @@
 	    ((ge) (if (>= (ev1 tree) (ev2 tree)) 1 0))
 	    ((equal) (if (= (ev1 tree) (ev2 tree)) 1 0))
 	    ((noteq) (if (= (ev1 tree) (ev2 tree)) 0 1))
-	    ((bw-or) (bitwise-ior (ev1 tree) (ev2 tree)))
-	    ((bw-xor) (bitwise-xor (ev1 tree) (ev2 tree)))
-	    ((bw-and) (bitwise-and (ev1 tree) (ev2 tree)))
+	    ((bitwise-not) (lognot (ev1 tree)))
+	    ((bitwise-or) (logior (ev1 tree) (ev2 tree)))
+	    ((bitwise-xor) (logxor (ev1 tree) (ev2 tree)))
+	    ((bitwise-and) (logand (ev1 tree) (ev2 tree)))
 	    ((or) (if (and (zero? (ev1 tree)) (zero? (ev2 tree))) 0 1))
 	    ((and) (if (or (zero? (ev1 tree)) (zero? (ev2 tree))) 0 1))
 	    ((cond-expr) (if (zero? (ev1 tree)) (ev3 tree) (ev2 tree)))
-	    ((ident) 0)
-	    (else (error "incomplete implementation"))))))
+	    ;; hacks for use in util2.scm:canize-enum-def-list:
+	    ;; If ident is defined then it should have already been expanded.
+	    ;; So then only enum defs remain which should be valid expressions.
+	    ((ident) (or (and=> (assoc-ref dict (tx1 tree)) string->number) 0))
+	    ((p-expr) (ev1 tree))
+	    ((cast) (ev2 tree))
+	    (else (error "incomplete eval-cpp-expr implementation"))))))
     (eval-expr tree)))
 
 ;;.@deffn {Procedure} rtokl->string reverse-token-list => string
@@ -243,7 +306,7 @@
   (define (add-chl chl stl)
     (if (null? chl) stl (cons (list->string chl) stl)))
 
-  ;;(simple-format #t "\nused=~S\n" used)
+  ;;(sferr "\nrtokl->string=~S\n" tokl)
   ;; Works like this: Scan through the list of tokens (key-val pairs or
   ;; lone characters).  Lone characters are collected in a list (@code{chl});
   ;; pairs are converted into strings and combined with list of characters
@@ -262,22 +325,22 @@
       (iter stl (cons (car tkl) chl) nxt (cdr tkl)))
      (else
       (pmatch tkl
-	(((ident . ,rval) dhash (ident . ,lval) . ,rest)
+	((($ident . ,rval) $dhash ($ident . ,lval) . ,rest)
 	 (iter stl chl nxt
-	       (acons 'ident (string-append lval rval) (list-tail tkl 3))))
-	(((ident . ,arg) hash . ,rest)
+	       (acons '$ident (string-append lval rval) (list-tail tkl 3))))
+	((($ident . ,arg) $hash . ,rest)
 	 (iter stl chl (string-append "\"" arg "\"") (list-tail tkl 2)))
-	(((ident . ,iden) (ident . ,lval) . ,rest)
+	((($ident . ,iden) ($ident . ,lval) . ,rest)
 	 (iter stl chl iden rest))
-	(((ident . ,iden) . ,rest)
+	((($ident . ,iden) . ,rest)
 	 (iter stl chl iden rest))
-	(((string . ,val) . ,rest)
-	 (iter stl (cons #\" chl) val (cons #\" rest)))
-	(((defined . ,val) . ,rest)
+	((($string . ,val) . ,rest)
+	 (iter stl (cons #\" chl) (esc-c-str val) (cons #\" rest)))
+	((($echo . ,val) . ,rest)
 	 (iter stl chl val rest))
-	((space space . ,rest)
+	(($space $space . ,rest)
 	 (iter stl chl nxt rest))
-	((space . ,rest)
+	(($space . ,rest)
 	 (iter stl (cons #\space chl) nxt rest))
 	((,asis . ,rest)
 	 (iter stl chl asis rest))
@@ -293,18 +356,35 @@
       (cond
        ((eof-object? ch)
 	(if no-ec
-	    (string-append "defined " (list->string (reverse chl)))
+	    (list->string (cons #\space (reverse chl)))
 	    (cpp-err "illegal argument to `defined'")))
        ((char-set-contains? c:ir ch)
 	(iter (cons ch chl) (read-char)))
        (no-ec
 	(unread-char ch)
-	(string-append "defined " (list->string (reverse chl))))
+	(list->string (cons #\space (reverse chl))))
        ((char=? #\) (skip-il-ws ch))
-	(string-append "defined" (list->string (reverse (cons #\) chl)))))
+	(list->string (reverse (cons #\) chl))))
        (else
 	(cpp-err "illegal argument to  `defined'"))))))
 
+;; must be (\s*<xxx>\s*) OR (\s*"xxx"\s*) => ("<xxx>") OR ("\"xxx\"")
+(define (scan-arg-literal)
+  (let ((ch (read-char)))
+    ;; if exit, then did not defined __has_include(X)=__has_include__(X)
+    (if (or (eof-object? ch) (not (char=? #\( ch))) (error "expedcting `('")))
+  (let iter ((chl '()) (ch (skip-il-ws (read-char))))
+    (cond
+     ((eof-object? ch) (cpp-err "illegal argument"))
+     ((char=? #\) ch)
+      (let iter2 ((res '()) (chl chl))
+	(cond
+	 ((null? chl)
+	  (string-append "(\"" (esc-c-str (list->string res)) "\")"))
+	 ((and (null? res) (char-whitespace? (car chl))) (iter2 res (cdr chl)))
+	 (else (iter2 (cons (car chl) res) (cdr chl))))))
+     (else (iter (cons ch chl) (read-char))))))
+
 (define (scan-cpp-input defs used end-tok)
   ;; Works like this: scan for tokens (comments, parens, strings, char's, etc).
   ;; Tokens are collected in a (reverse ordered) list (tkl) and merged together
@@ -314,24 +394,24 @@
   ;; as well as ABC/*foo*/(123,456).
 
   (define (trim-spaces tkl)
-      (if (and (pair? tkl) (eqv? 'space (car tkl)))
+      (if (and (pair? tkl) (eqv? '$space (car tkl)))
 	  (trim-spaces (cdr tkl))
 	  tkl))
 
   (define (finish rr tkl)
+    ;;(sferr "finish ~S ~S\n" rr tkl)
     (let* ((tkl (if end-tok (trim-spaces tkl) tkl))
 	   (repl (rtokl->string tkl)))
-      ;;(if (null? rr) (simple-format #t "finish ~S\n" tkl))
       (if (pair? rr)
-	  (expand-cpp-repl repl defs (append rr used)) ;; re-run
+	  (cpp-expand-text repl defs (append rr used)) ;; re-run
 	  repl)))
      
-  ;;(simple-format #t "scan-cpp-input end-tok=~S\n" end-tok)
+  ;;(sferr "scan-cpp-input end-tok=~S\n" end-tok)
   (let iter ((rr '())			; list symbols resolved
 	     (tkl '())			; token list of
 	     (lv 0)			; level
 	     (ch (skip-il-ws (read-char)))) ; next character
-    ;;(unless end-tok (simple-format #t "tkl=~S ch=~S\n" tkl ch))
+    ;;(unless end-tok (sferr "tkl=~S ch=~S\n" tkl ch))
     (cond
      ((eof-object? ch) (finish rr tkl))
      ((and (eqv? end-tok ch) (zero? lv))
@@ -340,23 +420,35 @@
       (unread-char ch) (finish rr tkl))
      ((or (char-set-contains? c:ws ch)	; whitespace
 	  (read-c-comm ch #f))		; comment
-      (iter rr (cons 'space tkl) lv (skip-il-ws (read-char))))
+      (iter rr (cons '$space tkl) lv (skip-il-ws (read-char))))
      ((read-c-ident ch) =>
       (lambda (iden)
-	(if (equal? iden "defined")
-	    (iter rr (acons 'defined (scan-defined-arg) tkl) lv (read-char))
-	    (let ((rval (expand-cpp-macro-ref iden defs used)))
-	      (if rval
-		  (iter #t (cons rval tkl) lv (read-char))
-		  (iter rr (acons 'ident iden tkl) lv (read-char)))))))
+	(cond
+	 ((string=? iden "defined")
+	  (iter rr
+		(acons '$echo (string-append iden (scan-defined-arg)) tkl)
+		lv (read-char)))
+	 ((member iden '("__has_include__" "__has_include_next__"))
+	  (cond
+	   ((scan-arg-literal) =>
+	    (lambda (arg)
+	      (iter rr (acons '$echo (string-append iden arg) tkl)
+		    lv (read-char))))
+	   (else 
+	    (iter rr (acons '$ident iden tkl) lv (read-char)))))
+	 (else
+	  (let ((rval (expand-cpp-macro-ref iden defs used)))
+	    (if rval
+		(iter #t (cons rval tkl) lv (read-char))
+		(iter rr (acons '$ident iden tkl) lv (read-char))))))))
      ((read-c-string ch) =>
-      (lambda (st) (iter rr (acons 'string (cdr st) tkl) lv (read-char))))
+      (lambda (pair) (iter rr (cons pair tkl) lv (read-char))))
      ((char=? #\( ch) (iter rr (cons ch tkl) (1+ lv) (read-char)))
      ((char=? #\) ch) (iter rr (cons ch tkl) (1- lv) (read-char)))
      (else
       (iter rr (cons ch tkl) lv (read-char))))))
 
-;; @deffn {Procedure} collect-args arglargd defs used => argd
+;; @deffn {Procedure} collect-args argl defs used => argd
 ;; Collect arguments to a macro which appears in C code.  If not looking at
 ;; @code{(} return @code{#f}, else scan and eat up to closing @code{)}.
 ;; If multiple whitespace characters are skipped at the front then only
@@ -395,7 +487,7 @@
 (define (px-cpp-ftn argd repl)
   (with-input-from-string repl
     (lambda ()
-      ;;(simple-format #t "px-cpp-ftn argd=~S repl=~S\n" argd repl)
+      ;;(sferr "px-cpp-ftn argd=~S repl=~S\n" argd repl)
       (px-cpp-ftn-1 argd))))
 
 (define (px-cpp-ftn-1 argd)
@@ -411,13 +503,13 @@
        ((char-set-contains? c:ws (car chl)) (iter (cdr chl)))
        (else chl))))
 
-  (define (mk-string str) (string-append "\"" str "\""))
+  (define (mk-string str) (string-append "\"" (esc-c-str str) "\""))
 
   (let iter ((stl '())			; string list
 	     (chl '())			; character list
 	     (nxt #f)			; next string after char list
 	     (ch (read-char)))		; next character
-    ;;(simple-format #t "px-1: stl=~S chl=~S nxt=~S ch=~S\n" stl chl nxt ch)
+    ;;(sferr "px-1: stl=~S chl=~S nxt=~S ch=~S\n" stl chl nxt ch)
     (cond
      (nxt (iter (cons nxt (ins-chl chl stl)) '() #f ch))
      ((eof-object? ch)
@@ -448,15 +540,6 @@
 ;; Identifiers in the list of strings @var{used} will not be expanded.
 ;; @end deffn
 (define* (cpp-expand-text text defs #:optional (used '()))
-  ;;(simple-format #t "cpp-expand-text ~S\n" text)
-  (with-input-from-string text
-    (lambda () (scan-cpp-input defs used #f))))
-
-;; @deffn {Procedure} expand-cpp-repl
-;; to be documented
-;; @end deffn
-(define (expand-cpp-repl text defs used)
-  ;;(simple-format #t "expand-cpp-repl text=~S used=~S\n" text used)
   (with-input-from-string text
     (lambda () (scan-cpp-input defs used #f))))
 
@@ -466,13 +549,13 @@
 ;; Evaluate CPP condition expression (text).
 ;; Undefined identifiers are replaced with @code{0}.
 ;; @end deffn
-(define (eval-cpp-cond-text text defs)
+(define* (eval-cpp-cond-text text defs #:key (inc-dirs '()))
   (with-throw-handler
    'cpp-error
    (lambda ()
      (let* ((rhs (cpp-expand-text text defs))
 	    (exp (parse-cpp-expr rhs)))
-       (eval-cpp-expr exp defs)))
+       (eval-cpp-expr exp defs #:inc-dirs inc-dirs)))
    (lambda (key fmt . args)
      (report-error fmt args)
      (throw 'c99-error "CPP error"))))
@@ -489,15 +572,22 @@
 ;; @end example
 ;; @end deffn
 (define* (expand-cpp-macro-ref ident defs #:optional (used '()))
+  ;;(sferr "x-mref ~S\n" ident)
   (let ((rval (assoc-ref defs ident)))
     (cond
      ((member ident used) #f)
-     ((string? rval) (expand-cpp-repl rval defs (cons ident used)))
+     ((string? rval)
+      (let* ((used (cons ident used))
+	     (repl (cpp-expand-text rval defs used)))
+	;;(sferr "rerun ~S\n" repl)
+	(if (ident-like? repl)
+	    (or (expand-cpp-macro-ref repl defs (cons repl used)) repl)
+	    repl)))
      ((pair? rval)
       (and=> (collect-args (car rval) defs used)
 	     (lambda (argd)
-	       (expand-cpp-repl (px-cpp-ftn argd (cdr rval))
-				defs (cons ident used)))))
+	       (let ((rez (px-cpp-ftn argd (cdr rval))))
+		 (cpp-expand-text rez defs (cons ident used))))))
      ((c99-std-val ident) => identity)
      (else #f))))
 
