@@ -1,22 +1,25 @@
-;;; lang/c99/mach.scm
-;;;
-;;; Copyright (C) 2015-2017 Matthew R. Wette
-;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; lang/c99/mach.scm
+;;
+;; Copyright (C) 2015-2017 Matthew R. Wette
+;;
+;; This program is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by 
+;; the Free Software Foundation, either version 3 of the License, or 
+;; (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;; C parser generator: based on ISO-C99; with comments and CPP statements
 
+;; but check this:
+;;   https://gcc.gnu.org/onlinedocs/gcc/C-Extensions.html#C-Extensions
+
 (define-module (nyacc lang c99 mach)
   #:export (c99-spec c99-mach dev-parse-c dev-parse-c99
 	    gen-c99-files gen-c99x-files)
@@ -52,6 +55,8 @@
      (constant ($$ `(p-expr ,$1)))
      (string-literal ($$ `(p-expr ,(tl->list $1))))
      ("(" expression ")" ($$ $2))
+     ("(" "{" block-item-list "}" ")"
+      ($$ `(stmt-expr (@ (extension "GNUC")) ,$3)))
      )
 
     (postfix-expression			; S 6.5.2
@@ -205,6 +210,8 @@
       ($$ (if (pair? $3)
 	      `(decl ,(tl->list $1) ,(list $3))
 	      `(decl ,(tl->list $1)))))
+     ;; macOS has things like: foo_t bar(int) __asm("bar");
+     ;; (declaration-specifiers darwin-asm ";")
      )
 
     (declaration-specifiers		; S 6.7
@@ -229,7 +236,9 @@
 
     (init-declarator			; S 6.7
      (declarator ($$ `(init-declr ,$1)))
+     (declarator asm-expression ($$ `(init-declr ,$1)))
      (declarator "=" initializer ($$ `(init-declr ,$1 ,$3)))
+     (declarator asm-expression "=" initializer ($$ `(init-declr ,$1 ,$4)))
      )
 
     (storage-class-specifier		; S 6.7.1
@@ -385,7 +394,6 @@
     (direct-declarator			; S 6.7.6
      (identifier ($$ $1))
      ("(" declarator ")" ($$ `(scope ,$2)))
-
      (direct-declarator
       "[" type-qualifier-list assignment-expression "]"
       ($$ `(array-of ,$1 ,$3 ,$4)))
@@ -412,14 +420,18 @@
      )
 
     (pointer				; S 6.7.6
-     ("*" type-qualifier-list ($$ `(pointer ,(tl->list $2))))
-     ("*" ($$ '(pointer)))
      ("*" type-qualifier-list pointer ($$ `(pointer ,(tl->list $2) ,$3)))
+     ("*" type-qualifier-list ($$ `(pointer ,(tl->list $2))))
      ("*" pointer ($$ `(pointer ,$2)))
+     ("*" ($$ '(pointer)))
      )
 
-    ;; But put in tree as decl-spec-list
     (type-qualifier-list
+     (type-qualifier ($$ (make-tl 'type-qual-list $1)))
+     (type-qualifier-list type-qualifier ($$ (tl-append $1 $2)))
+     )
+    ;; WAS: But put in tree as decl-spec-list
+    #;(type-qualifier-list
      (type-qualifier ($$ (make-tl 'decl-spec-list $1)))
      (type-qualifier-list type-qualifier ($$ (tl-append $1 $2)))
      )
@@ -457,8 +469,8 @@
      )
 
     (abstract-declarator		; S 6.7.6
-     (pointer ($$ `(abs-declr ,$1)))
      (pointer direct-abstract-declarator ($$ `(abs-declr ,$1 ,$2)))
+     (pointer ($$ `(abs-declr ,$1)))
      (direct-abstract-declarator ($$ `(abs-declr ,$1)))
      )
 
@@ -466,7 +478,7 @@
      ("(" abstract-declarator ")" ($$ `(declr-scope ,$2)))
      (direct-abstract-declarator
       "[" type-qualifier-list assignment-expression "]"
-      ($$ `(declr-array ,$3 ,$4)))
+      ($$ `(declr-array ,$1 ,$3 ,$4)))
      (direct-abstract-declarator
       "[" type-qualifier-list "]"
       ($$ `(declr-array ,$1 ,$3)))
@@ -500,13 +512,13 @@
      ("[" type-qualifier-list "static" assignment-expression "]"
       ($$ `(declr-anon-array
 	    ,(tl->list (tl-insert '(stor-spec "static") $2)) ,$4)))
-     (direct-abstract-declarator "[" "*" "]" ($$ `(declr-STAR ,$1)))
-     ("[" "*" "]" ($$ '(declr-STAR)))
+     (direct-abstract-declarator "[" "*" "]" ($$ `(declr-star ,$1)))
+     ("[" "*" "]" ($$ '(declr-star)))
      (direct-abstract-declarator "(" parameter-type-list ")"
-				 ($$ `(declr-fctn ,$1 ,(tl->list $3))))
-     (direct-abstract-declarator "(" ")" ($$ `(declr-fctn ,$1)))
-     ("(" parameter-type-list ")" ($$ `(declr-anon-fctn ,(tl->list $2))))
-     ("(" ")" ($$ '(declr-anon-fctn)))
+				 ($$ `(abs-ftn-declr ,$1 ,(tl->list $3))))
+     (direct-abstract-declarator "(" ")" ($$ `(abs-ftn-declr ,$1)))
+     ("(" parameter-type-list ")" ($$ `(anon-ftn-declr ,(tl->list $2))))
+     ("(" ")" ($$ '(anon-ftn-declr)))
      )
 
     ;;typedef-name must be hacked w/ the lexical analyzer
@@ -536,8 +548,8 @@
      )
 
     (designator
-     ("[" constant-expression "]" ($$ (list 'array-dsgr $2)))
-     ("." identifier ($$ (list 'sel-dsgr $2)))
+     ("[" constant-expression "]" ($$ `(array-dsgr ,$2)))
+     ("." identifier ($$ `(sel-dsgr ,$2)))
      )
 
     ;; statements
@@ -548,6 +560,7 @@
      (selection-statement)
      (iteration-statement)
      (jump-statement)
+     (asm-statement)
      (cpp-statement)
      )
 
@@ -609,6 +622,40 @@
      ("return" ";" ($$ `(return (expr))))
      )
 
+    (asm-statement			; NOT part of C99
+     (asm-expression ";"))
+    (asm-expression			; NOT part of C99
+     ("asm" "(" string-literal ")"
+      ($$ `(asm-expr (@ (extension "GNUC")) ,$3)))
+     ("asm" "(" string-literal asm-outputs ")"
+      ($$ `(asm-expr (@ (extension "GNUC")) ,$3 ,(tl->list $4))))
+     ("asm" "(" string-literal asm-outputs asm-inputs ")"
+      ($$ `(asm-expr (@ (extension "GNUC")) ,$3 ,(tl->list $4) ,(tl->list $5))))
+     ("asm" "(" string-literal asm-outputs asm-inputs asm-clobbers ")"
+      ($$ `(asm-expr (@ (extension "GNUC"))
+		     ,$3 ,(tl->list $4) ,(tl->list $5) ,(tl->list $6)))))
+    (asm-outputs
+     (":" ($$ (make-tl 'asm-outputs)))
+     (":" asm-output ($$ (make-tl 'asm-outputs $2)))
+     (asm-outputs "," asm-output ($$ (tl-append $1 $3))))
+    (asm-output
+     (string-literal "(" identifier ")" ($$ `(asm-operand ,$1 ,$3)))
+     ("[" identifier "]" string-literal "(" identifier ")"
+      ($$ `(asm-operand ,$2 ,$4 ,$6))))
+    (asm-inputs
+     (":" ($$ (make-tl 'asm-inputs)))
+     (":" asm-input ($$ (make-tl 'asm-inputs $2)))
+     (asm-inputs "," asm-input ($$ (tl-append $1 $3))))
+    (asm-input
+     (string-literal "(" expression ")" ($$ `(asm-operand ,$1 ,$3)))
+     ("[" identifier "]" string-literal "(" expression ")"
+      ($$ `(asm-operand ,$2 ,$4 ,$6))))
+    (asm-clobbers
+     (":" ($$ (make-tl 'asm-clobbers)))
+     (":" string-literal ($$ (make-tl 'asm-clobbers $2)))
+     (asm-clobbers "," string-literal ($$ (tl-append $1 $3)))
+     )
+
     ;; external definitions
     (translation-unit			; S 6.9
      (external-declaration-list ($$ (tl->list $1)))
@@ -634,7 +681,7 @@
       ($$ `(extern-block (extern-begin ,$2)
 			 ,@(sx-tail (tl->list $5) 1)
 			 (extern-end))))
-     (";" ($$ `(decl (@ (extension . "GNU C")))))
+     (";" ($$ `(decl (@ (extension "GNUC")))))
      )
     
     (function-definition
@@ -668,7 +715,7 @@
     (code-comment ($code-comm ($$ `(comment ,$1))))
     (lone-comment ($lone-comm ($$ `(comment ,$1))))
     (cpp-statement ('cpp-stmt ($$ `(cpp-stmt ,$1))))
-    (pragma ('cpp-pragma))
+    (pragma ("_Pragma" "(" string-literal ")" ($$ `(pragma ,(tl->list $3)))))
     )))
 
 
