@@ -1,19 +1,19 @@
-;;; nyacc/lang/c99/pprint.scm
-;;;
-;;; Copyright (C) 2015-2017 Matthew R. Wette
-;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; nyacc/lang/c99/pprint.scm
+;;
+;; Copyright (C) 2015-2017 Matthew R. Wette
+;;
+;; This program is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by 
+;; the Free Software Foundation, either version 3 of the License, or 
+;; (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (nyacc lang c99 pprint)
   #:export (pretty-print-c99)
@@ -77,14 +77,39 @@
       ((#\\) "\\")
       (else scm-chr-str))))
 
-(define protect-expr? (make-protect-expr op-prec op-assc))
+;; @deffn {Procedure} char->hex-list ch seed
+;; to be documented
+;; @end deffn
+(define (char->hex-list ch seed)
+  (define (itox ival) (string-ref "0123456789ABCDE" ival))
+  (let iter ((res seed) (ix 8) (val (char->integer ch)))
+    (cond
+     ((zero? ix) (cons* #\\ #\U res))
+     ((and (zero? val) (= ix 4)) (cons* #\\ #\u res))
+     (else
+      (iter (cons (itox (remainder val 16)) res) (1- ix) (quotient val 16))))))
 
-;; @deffn pretty-print-c99 tree [port] [options]
+;; @deffn {Procedure} scmstr->c str
+;; to be documented
+;; @end deffn
+(define (scmstr->c str)
+  (list->string
+   (string-fold-right
+    (lambda (ch chl)
+      (cond
+       ((char-set-contains? char-set:printing ch) (cons ch chl))
+       ((char=? ch #\nul) (cons* #\\ #\0 chl))
+       (else (char->hex-list ch chl))))
+    '() str)))
+    
+;;(define protect-expr? (make-protect-expr op-prec op-assc))
+
+;; @deffn {Procedure} pretty-print-c99 tree [port] [options]
 ;; Convert and print a C99 sxml tree to the current output port.
-;; The optional keyword argument @code{#:indent-level} provides the
+;; The optional keyword argument @code{#:basic-offset} provides the
 ;; indent level, with default of 2.
 ;; @table code
-;; @item #:indent-level <level>
+;; @item #:basic-offset <n>
 ;; indent level for C code
 ;; @item #:per-line-prefix <string>
 ;; string
@@ -94,11 +119,11 @@
 ;; @end deffn
 (define* (pretty-print-c99 tree
 			   #:optional (port (current-output-port))
-			   #:key (indent-level 2) ugly per-line-prefix)
+			   #:key ugly per-line-prefix (basic-offset 2))
 
   (define fmtr
     ((if ugly make-pp-formatter/ugly make-pp-formatter)
-     port #:per-line-prefix per-line-prefix))
+     port #:per-line-prefix per-line-prefix #:basic-offset basic-offset))
   (define (push-il)(fmtr 'push))
   (define (pop-il) (fmtr 'pop))
 
@@ -128,6 +153,8 @@
        (simple-format #t "\n*** pprint/cpp-ppx: NO MATCH: ~S\n" tree)))
     (fmtr 'nlin))
 
+  (define protect-expr? (make-protect-expr op-prec op-assc))
+  
   (define (unary/l op rep rval)
     (sf rep)
     (if (protect-expr? 'rt op rval)
@@ -188,12 +215,32 @@
       ((string . ,value-l)
        (pair-for-each
 	(lambda (pair)
-	  (sf "~S" (car pair))
+	  (sf "~S" (scmstr->c (car pair)))
 	  (if (pair? (cdr pair)) (sf " ")))
 	value-l))
 
-      ((comment ,text) (sf "/*~A*/\n" text))
-
+      ((comment ,text)
+       ;; Comments will look funny when indent for input and output
+       ;; are different since multi-line comments will get hosed.
+       (for-each (lambda (l) (sf (scmstr->c l)) (sf "\n"))
+		 (string-split (string-append "/*" text "*/") #\newline))
+       ;; TODO: Since parser now removes prefix, I need to add it back in here.
+       ;; needed:
+       ;; 1) (get-col) to get column
+       ;; 2) (indent-to-col col) to do indents for each line
+       #;(let ((col (get-col))
+	     (lines (string-split text #\newline))
+	     (ind (mk-ind-to-col col)))
+	 (sf "/*") (sf (car lines))
+	 (pair-for-each
+	  (lambda (pair)
+	    (sf "\n")
+	    (if (pair? (cdr pair)) (sf ind))
+	    (sf (cdr pair)))
+	  (cdr lines))
+	 (sf "*/"))
+       )
+      
       ((scope ,expr) (sf "(") (ppx expr) (sf ")"))
       
       ((array-ref ,dim ,expr)
@@ -346,6 +393,9 @@
       ((comp-declr ,declr) (comp declr #f))
       ((param-declr ,declr) (comp declr #f))
 
+      ((bit-field ,ident ,expr)
+       (ppx ident) (sf " : ") (ppx expr))
+
       ((type-spec ,arg)
        (case (sx-tag arg)
 	 ((fixed-type) (sf "~A" (sx-ref arg 1)))
@@ -382,10 +432,12 @@
        (for-each ppx defns)
        (pop-il) (sf "}"))
 
-      ((enum-defn (ident ,name) (p-expr (fixed ,value)))
-       (sf "~A = ~A,\n" name value))
-      ((enum-defn (ident ,name) (neg (p-expr (fixed ,value))))
-       (sf "~A = -~A,\n" name value))
+      ((enum-defn (ident ,name) (comment ,comment))
+       (sf "~A, " name) (ppx `(comment ,comment)) (sf "\n"))
+      ((enum-defn (ident ,name) ,expr ,comment)
+       (sf "~A = " name) (ppx expr) (sf ", ") (ppx comment) (sf "\n"))
+      ((enum-defn (ident ,name) ,expr)
+       (sf "~A = " name) (ppx expr) (sf ",\n"))
       ((enum-defn (ident ,name))
        (sf "~A,\n" name))
 
@@ -399,6 +451,15 @@
       ((pointer ,one) (sf "*") (ppx one))
       ((pointer ,one ,two) (sf "*") (ppx one) (ppx two))
 
+      ((type-qual-list . ,tql)		; see decl-spec-list
+       (pair-for-each
+	(lambda (dsl)
+	  (case (sx-tag (car dsl))
+	    ((type-qual) (sf "~A" (sx-ref (car dsl) 1)))
+	    (else (sf "[?:~S]" (car dsl))))
+	  (if (pair? (cdr dsl)) (sf " ")))
+	tql))
+
       ((array-of ,dir-declr ,arg)
        (ppx dir-declr) (sf "[") (ppx arg) (sf "]"))
       ((array-of ,dir-declr)
@@ -416,6 +477,29 @@
       ((abs-declr ,pointer ,dir-abs-declr) (ppx pointer) (ppx dir-abs-declr))
       ((abs-declr ,one-of-above) (ppx one-of-above))
 
+      ;; declr-scope
+      ((declr-scope ,abs-declr)
+       (sf "(") (ppx abs-declr) (sf ")"))
+	
+      ;; declr-array dir-abs-declr
+      ;; declr-array dir-abs-declr assn-expr
+      ;; declr-array dir-abs-declr type-qual-list
+      ;; declr-array dir-abs-declr type-qual-list assn-expr
+      ((declr-array ,dir-abs-declr)
+       (ppx dir-abs-declr) (sf "[]"))
+      ((declr-array ,dir-abs-declr ,arg2)
+       (ppx dir-abs-declr) (sf "[") (ppx arg2) (sf "]"))
+      ((declr-array ,dir-abs-declr ,arg2 ,arg3)
+       (ppx dir-abs-declr) (sf "[") (ppx arg2) (sf " ") (ppx arg3) (sf "]"))
+	
+      ;; declr-anon-array
+      ;; declr-STAR
+
+      ;; abs-ftn-declr
+      ((abs-ftn-declr ,dir-abs-declr ,param-type-list)
+       (ppx dir-abs-declr) (sf "(") (ppx param-type-list) (sf ")"))
+      ;; anon-ftn-declr
+
       ((compd-stmt (block-item-list . ,items))
        (sf "{\n") (push-il) (for-each ppx items) (pop-il) (sf "}\n"))
       ((compd-stmt-no-newline (block-item-list . ,items))
