@@ -68,8 +68,8 @@
 	    ;; munging
 	    expand-typerefs
 	    stripdown
-	    udecl->mspec udecl->mspec/comm
 	    udecl-rem-type-qual specl-rem-type-qual
+	    udecl->mspec udecl->mspec/comm mspec->udecl
 
 	    unwrap-decl
 	    canize-enum-def-list
@@ -166,6 +166,30 @@
      (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
     ;;
     (,otherwise #f)))
+;; @deffn {Procedure} pointer-stor-declr? declr => #t|#f
+;; @deffnx {Procedure} pointer-pass-declr? declr => #t|#f
+;; This predicate determines if the declarator is implemented as a pointer.
+;; That is, it is an explicit pointer, an array (ERROR), or a function.
+;; @end deffn
+;;(define (pointer-stor-declr? declr)
+(define (pointer-pass-declr? declr)
+  (sxml-match declr
+    (#f #f)
+    ((init-declr ,declr) (pointer-declr? declr))
+    ((comp-declr ,declr) (pointer-declr? declr))
+    ((param-declr ,declr) (pointer-declr? declr))
+    ;;
+    ((ptr-declr ,pointer ,dir-declr) #t)
+    ((array-of . ,rest) #t)
+    ((ftn-declr . ,rest) #t)
+    ((abs-declr (pointer . ,r1) . ,r2) #t)
+    ;;
+    ((init-declr-list ,declrs ...)
+     (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
+    ((comp-declr-list ,declrs ...)
+     (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
+    ;;
+    (,otherwise #f)))
 
 ;; Use the term @dfn{udecl}, or unit-declaration, for a declaration which has
 ;; only one decl-item.  That is where,
@@ -211,7 +235,8 @@
 ;; @deffnx {Procedure} c99-trans-unit->udict/deep tree [seed]
 ;; Convert a C parse tree into a assoc-list of global names and definitions.
 ;; This will unwrap @code{init-declr-list} into list of decls w/
-;; @code{init-declr}.  The declarations come reversed from order in file!
+;; @code{init-declr}.
+;; The declarations come reversed from order in file!
 ;; @example
 ;; BUG: need to add struct and union defn's: struct foo { int x; };
 ;; how to deal with this
@@ -566,6 +591,7 @@
     ((ptr-declr ,pointer ,dir-declr) (declr-ident dir-declr))
     ((ftn-declr ,dir-declr ,rest ...) (declr-ident dir-declr))
     ((scope ,declr) (declr-ident declr))
+    ((bit-field ,ident . ,rest) ident)
     (,otherwise (throw 'util-error "c99/util2: unknown declarator: " declr))))
 
 ;; @deffn {Procedure} declr-id decl => "name"
@@ -800,17 +826,14 @@
 	 (name (sx-ref tspec 1)))	; e.g., "foo_t"
     (case class
       ((typename)
-       (when #f ;;(string=? name "sqlite3_vfs")
-	 (sferr "\n")
-	 (pperr specl) (pperr declr)
-	 (sferr "pointer-declr => ~S\n" (pointer-declr? declr))
-	 ;;(error "stopping")
-	 )
        (cond
 	((member name keep)		; keeper; don't expand
 	 (values specl declr))
-	((pointer-declr? declr)		; pointer, replace with void*
-	 (values (replace-type-spec specl '(type-spec (void))) declr))
+	#;((pointer-declr? name)		; replace with void*
+	 (let ((specl (replace-type-spec specl '(type-spec (void)))))
+	   (call-with-values
+	       (lambda () (splice-typename specl declr name udict)))
+	   (lambda (specl declr) (re-expand specl declr))))
 	(else				; expand
 	 (call-with-values
 	     (lambda () (splice-typename specl declr name udict))
@@ -929,6 +952,7 @@
     (sxml-match declr
       (#f declr)
       ((ident ,name) declr)
+      ((bit-field . ,rest) declr)
       
       ((init-declr ,declr1 . ,rest)
        (let ((xdeclr (fix-declr declr1)))
@@ -1043,6 +1067,9 @@
 		 (split-adecl adecl))
 		((repl-specl repl-declr)
 		 (expand-specl-typerefs orig-specl orig-declr udict keep)))
+    ;;(sferr "orig-specl, orig-declr, repl-specl, repl-declr\n")
+    ;;(pperr orig-specl) (pperr orig-declr)
+    ;;(pperr repl-specl) (pperr repl-declr)
     (let ((repl-declr (fix-declr repl-declr)))
       (if (and (eq? orig-specl repl-specl)
 	       (eq? orig-declr repl-declr))
@@ -1340,7 +1367,7 @@
 ;; @end deffn
 (define* (udecl->mspec decl #:key abs-ident)
 
-  (define (cnvt-array-size size-spec)
+  (define (cnvt-size-expr size-spec)
     (with-output-to-string (lambda () (pretty-print-c99 size-spec))))
 
   (define (unwrap-specl specl)
@@ -1373,7 +1400,7 @@
        (unwrap-declr item #:const const))
 
       ((array-of ,dir-declr ,size)
-       (cons `(array-of ,(cnvt-array-size size)) (unwrap-declr dir-declr)))
+       (cons `(array-of ,(cnvt-size-expr size)) (unwrap-declr dir-declr)))
       ((array-of ,dir-declr)
        (cons `(array-of) (unwrap-declr dir-declr)))
 
@@ -1419,6 +1446,9 @@
        ;; TODO: deal with "const" type-qualifier
        (cons `(array-of ,assn-expr) (unwrap-declr dir-abs-declr)))
 
+      ((bit-field (ident ,name) ,size)
+       (list `(bit-field ,(cnvt-size-expr size)) name))
+
       ((comp-declr ,item) (unwrap-declr item))
       ((param-declr ,item) (unwrap-declr item))
 
@@ -1450,6 +1480,35 @@
 	 (spec (udecl->mspec decl)))
     (cons* (car spec) comm (cdr spec))))
 
+
+;; @deffn {Procedure} mspec->udecl mspec xxx
+;; needed for xxx
+;; @end deffn
+(define* (mspec->udecl mspec)
+
+  (define (make-udecl types declr)
+    `(udecl (decl-spec-list (type-spec ,types)) (init-declr ,declr)))
+
+  (define (doit declr mspec-tail)
+    (pmatch mspec-tail
+      (((fixed-type ,name)) (make-udecl (car mspec-tail) declr))
+      (((float-type ,name)) (make-udecl (car mspec-tail) declr))
+      (((typename ,name)) (make-udecl (car mspec-tail) declr))
+      (((void)) (make-udecl (car mspec-tail) declr))
+      
+      (((pointer-to) . ,rest)
+       (doit `(ptr-declr (pointer) ,declr) rest))
+       
+      (,otherwise
+       (sferr "util2/mspec->udecl missed:\n")
+       (pperr otherwise)
+       (error "util2/mspec->udecl failed")
+       #f)))
+
+  (let ((name (car mspec))
+	(rest (cdr mspec)))
+    (doit `(ident ,name) rest)))
+
 ;; === deprecated ====================
 
 (define tree->udict c99-trans-unit->udict)
